Option Explicit     ' All variables must be declared using the 'Dim' statement
Scripting.DontExit = True ' Hang around to listen to events

'---------------------------------------
'Begin Main program
'---------------------------------------
' The units of all the constants below are in mils
Dim dielectricBoxX		'width of the box for the dielectric layers
Dim dielectricBoxY		'height of the box for the dielectric layers
Dim traceBoxX				'width of the box for trace layers (two each layer)
Dim traceBoxY				'height of the box for trace layers
Dim traceBox1OffsetX		'start position of first trace box
Dim traceBox2OffsetX		'start position of second trace box
Dim planeBoxX				'width of the box for plane layers
Dim planeBoxY				'height of the box for plane layers
Dim textOffsetX			'X offset for text for each layer
Dim textOffsetYdie		'Y offset for text for dielectric layer
Dim textOffsetYsm			'Y offset for text for solder mask layer
Dim textOffsetYcond		'Y offset for text for conductive layers
Dim textHeight			'height of text
Const userLayerName = "Layer Stackup"
Dim stepCondY				'spacing above conductive layer
Dim stepDieY				'spacing above dielectric layer
Dim stepTitleY			'spacing in title area
Dim titleTextOffestX		'start position of title text
Dim sm_box				'height of the box for soldermask layers

Dim colorPatternObj
Dim currentCondLayer
Dim currentDieLayer
Dim cUsrLyrGrfx, cUsrLyrTxts
Dim dontexit
Dim GetPointCmd
Dim i
Dim lyrObj
Dim lyrObjColl
Dim numOfPnts : numOfPnts = 5     ' 5 points for a rectangle
Dim app
Dim doc
Dim pntsArr
Dim rectObj
Dim textStr
Dim totalThickness : totalThickness = 0
Dim units
Dim usrLyrObj
Dim globalDisplayControl
Dim tagMotion : tagMotion = -1
dim font
dim unit_str

dim prog_id

get_id()
Set app = GetObject(, "MGCPCB.ExpeditionPCBApplication")
Set doc = GetLicensedDoc(app)
set gui = app.gui




' below are variables for the current X and Y coordinates for where we are going 
' to draw the boxes for the stackup and text. Most likely the X coordinates will
' not change since we are just going to up to draw the next box. For this example
' the origin of 0, 0 will be used as the starting X and Y coordinates.
Dim curX : curX = 0   
Dim curY : curY = 0   


' License the document
If (ValidateServer(doc) = 1) Then
    
    ' add a reference to the MGCPCB type library in order to use enums
    Scripting.AddTypeLibrary("MGCPCB.Expeditionapplication")
	
	Call set_units()
    units = doc.currentunit
	
	font = "Arial"
	
	select case doc.currentunit
		Case 2
			unit_str = "th"
		Case 3
			unit_str = "in"
		Case 4
			unit_str = "mm"
		Case 5
			unit_str = "um"
	End Select
    
    ' register a cmd so we can get a mouse point
    Set GetPointCmd = app.Gui.RegisterCommand("GetPoint")
    
    ' attach the events to the command
    Scripting.AttachEvents GetPointCmd, "getpntcmd"
	Call DrawMotionGraphics(doc.ActiveView.MousePositionX, doc.ActiveView.MousePositionY)
    
    ' set the prompt field in the status bar
    app.Gui.StatusBarText "Enter point for lower left corner of stackup...", _
    	epcbStatusFieldPrompt
    	
Else
    Msgbox("Could not validate the server. Exiting program.")
End If
'---------------------------------------
'End Main program
'---------------------------------------


'-------------------------------------------------------------------------------------------------------------------------------------------------------------------
'Local functions
'---------------------------------------
Function DrawMotionGraphics(x, y)
    'Call ClearMotionGraphics()
	'tagMotion = doc.ActiveView.MotionGfxGetTag()
   	'Call GenerateStackupGraphics(Nothing, x, y)
    'doc.ActiveView.MotionGfxDrawAnyUndrawn
End Function

Function ClearMotionGraphics()
	doc.ActiveView.MotionGfxDeleteByTag(tagMotion)
	tagMotion = -1
End Function

Function getpntcmd_OnMouseClk(button, flags, dx, dy)
    If button = epcbMouseButtonLeft Then
        Call ClearMotionGraphics()
		Call GenerateStackupGraphicsOnUserLayer(dx, dy)
        getpntcmd_OnMouseClk = True
		Gui.TerminateCommand
    End If
End Function

Function getpntcmd_OnMouseMove(button, flags, dx, dy)
	gui.StatusBarText "x: " & dx & "    y: " & dy
End Function

Function GenerateStackupGraphics(usrLyrObj, x, y)

    curX = x
    curY = y

    ' set the conductive and dielectric layer counts
    currentCondLayer = doc.LayerCount
    currentDieLayer = currentCondLayer + 1
        
    ' get the layer objects collection
    Set lyrObjColl = doc.LayerStack(True)
    
    ' since we are starting at the bottom and working our way up
    ' loop through the collection backward in a for loop
    For i = lyrObjColl.Count to 1 Step -1
        ' clear the text string for this layer
        textStr = ""
        
        ' get the layer object from the collection
        Set lyrObj = lyrObjColl.Item(i)
        
        ' set the text string
		Select Case i
			Case 1
				textStr = "     " & lyrObj.LayerProperties.Thickness & " " & unit_str &  ", Solder Mask Top"
			Case lyrObjColl.Count
				textStr = "     " & lyrObj.LayerProperties.Thickness & " " & unit_str &  ", Solder Mask Bottom"
			Case else
				textStr = "     " & lyrObj.LayerProperties.Thickness & " " & unit_str & ", " & lyrObj.LayerProperties.Description
		End Select
        
        
        ' add the layer thickness to the total
        totalThickness = totalThickness + _
        	lyrObj.LayerProperties.Thickness
        
        ' if this a dielectric layer then draw its layer and text
        If lyrObj.Type = epcbLayerTypeInsulation Then
        
			Select Case i
				Case 1
					Call DrawSolderMaskLayer(lyrObj, usrLyrObj,curX, curY, units, textStr)
				Case lyrObjColl.Count
					Call DrawSolderMaskLayer(lyrObj, usrLyrObj,curX, curY, units, textStr)
				Case else
					Call DrawDielectricLayer(lyrObj, usrLyrObj,curX, curY, units, textStr)
			End Select
        	
            ' decrement the layer counter
            currentDieLayer = currentDieLayer - 1
            
            ' step the Y position
            curY = curY + stepCondY
        ' if this is a conductive layer then draw is layer and text
        ElseIf lyrObj.Type = epcbLayerTypeConductor Then            
            ' if its a signal layer then draw two small boxes
            If lyrObj.ConductorType = epcbConductorTypeSignal Then
        		Call DrawSignalLayer(lyrObj, usrLyrObj, curX, curY, units)                
            ' if its a plane layer then draw one rectangle
            ElseIf lyrObj.ConductorType = epcbConductorTypePlane Then
        		Call DrawPlaneLayer(lyrObj, usrLyrObj, curX, curY, units)
            End If

            ' decrement the layer counter
            currentCondLayer = currentCondLayer - 1
            
            ' step the Y position
            curY = curY + stepDieY
        End If
    Next

End Function




Function GenerateStackupGraphicsOnUserLayer(x, y)
    curX = x
    curY = y
	ConfigureUserLayer

    ' start the transaction
    doc.TransactionStart
    
    totalThickness = 0.0
    
    Call GenerateStackupGraphics(usrLyrObj, x, y)

    
    ' step the Y position
    curY = curY + stepTitleY
    
    ' now we need to add the header text
    textstr = "Total Board Thickness: " & totalThickness
    
    ' put the text string on the user layer
    Call doc.PutUserLayerText(textStr, _
                            curX + titleTextOffestX, _
                            curY, _
                            usrLyrObj, _
                            textHeight,  _
                            0, _
                            5, _
                            font, _
                            , epcbJustifyHCenter, _
                            epcbJustifyBottom, _
                            Nothing, _
                            units)
    
    ' step the Y position
    curY = curY + stepTitleY

    ' now we need to add the header text
    textstr = "Layer Stackup"
    
    ' put the text string on the user layer
    Call doc.PutUserLayerText(textStr, _
                            curX + titleTextOffestX, _
                            curY, _
                            usrLyrObj, _
                            textHeight,  _
                            0, _
                            5, _
                            font, _
                            epcbTextAttrIsUnderlined , _
                            epcbJustifyHCenter, _
                            epcbJustifyBottom, _
                            Nothing, _
                            units)
    
    ' clear the prompt field in the status bar
    Call app.Gui.StatusBarText("", epcbStatusFieldPrompt)

    ' end the transaction
    doc.TransactionEnd
End Function 



Function ConfigureUserLayer()
    ' if the user layer exists already, delete all graphics on that layer
    Set usrLyrObj = doc.FindUserLayer(userLayerName)
    
    ' if the layer doesn't exist, create it, else delete all the data off the layer
    If usrLyrObj Is Nothing Then
        ' push the output mask to a user layer; create a user layer
         Set usrLyrObj = doc.SetupParameter.PutUserLayer(userLayerName)
    Else
        ' delete all the user layer gfx on target layer
        Set cUsrLyrGrfx = doc.UserLayerGfxs(, userLayerName)
        cUsrLyrGrfx.Delete
        Set cUsrLyrTxts = doc.UserLayerTexts(, userLayerName)
        cUsrLyrTxts.Delete
    End If

    ' ensure the user layer is turned on
    doc.ActiveView.DisplayControl.UserLayer(userLayerName) = True
    
    ' get the color pattern object for this user layer
    Set globalDisplayControl = doc.ActiveView.DisplayControl.Global
    Set colorPatternObj = globalDisplayControl.UserLayerColor(userLayerName)
    
    ' set it to the pattern we want
    colorPatternObj.Index = 13
    
    ' now we have to set the UserLayerColor object to the colorpattern 
    ' object we just modified
    Set globalDisplayControl.UserLayerColor(userLayerName) = colorPatternObj
End Function

Function DrawDielectricLayer(objLayer, objUserLayer, originX, originY, units, text_string)
	Call DrawRectangle(objUserLayer, originX, originY, dielectricBoxX, dielectricBoxY, False, units)
	Call DrawText(objUserLayer, text_string, originX + textOffsetX, originY + textOffsetYdie, units)
End Function

Function DrawSolderMaskLayer(objLayer, objUserLayer, originX, originY, units, text_string)
	Call DrawRectangle(objUserLayer, originX, originY, dielectricBoxX, sm_box, False, units)
	Call DrawText(objUserLayer, text_string, originX + textOffsetX, originY + textOffsetYsm, units)
End Function

Function DrawSignalLayer(objLayer, objUserLayer, originX, originY, units)
	Call DrawRectangle(objUserLayer, originX + traceBox1OffsetX, originY, traceBoxX, traceBoxY, True, units)
	Call DrawRectangle(objUserLayer, originX + traceBox2OffsetX, originY, traceBoxX, traceBoxY, True, units)
	Call DrawText(objUserLayer, GetLayerDescription(objLayer), originX + textOffsetX, originY + textOffsetYcond, units)
End Function

Function DrawPlaneLayer(objLayer, objUserLayer, originX, originY, units)
	Call DrawRectangle(objUserLayer, originX, originY, planeBoxX, planeBoxY, True, units)
	Call DrawText(objUserLayer, GetLayerDescription(objLayer), originX + textOffsetX, originY + textOffsetYcond, units)
End Function

Function GetLayerDescription(objLayer)
        GetLayerDescription = objLayer.LayerProperties.Thickness & " th, " & _
        	objLayer.LayerProperties.Description
End Function

Function DrawText(objUserLayer, strText, originX, originY, units)
	' If the user layer passed in is Nothing, then draw motion graphics
	If objUserLayer Is Nothing Then
		Call doc.ActiveView.MotionGfxPutText(strText, _
                                     originX, _
                                     originY, _
                                     epcbMotionGfxPointMoveXY, _
                                     textHeight,  _
                                     0, _
                                     5, _
                                     font, epcbTextAttrIsBold, _
                                     epcbJustifyLeft, _
                                     epcbJustifyVCenter, _
                                     tagMotion, _
                                     Nothing, _
                                     True, _
                                     units, epcbAngleUnitDegrees)
	Else
		Call doc.PutUserLayerText(strText, _
                                     originX, _
                                     originY, _
                                     objUserLayer, _
                                     textHeight,  _
                                     0, _
                                     5, _
                                     font, , _
                                     epcbJustifyLeft, _
                                     epcbJustifyVCenter, _
                                     Nothing, _
                                     units)
    End If
End Function

Function DrawRectangle(objUserLayer, originX, originY, width, height, bFill, units)
	' If the user layer passed in is Nothing, then draw motion graphics
	If objUserLayer Is Nothing Then
		Call doc.ActiveView.MotionGfxPutRect(originX, originY, epcbMotionGfxPointMoveXY, _
			originX + width, originY + height, epcbMotionGfxPointMoveXY, _
			tagMotion, 1, Nothing, units)
	Else
	   ' get the rectangular points array for this layer
       pntsArr = GetRectPntsArray(originX, originY, width, height)
            
       ' put the rectangle on the user layer
       Set rectObj = doc.PutUserLayerGfx(objUserLayer, 0, _
            numOfPnts, pntsArr, bFill, Nothing, units)
    End If
End Function

Function GetRectPntsArray(ByVal originX, ByVal originY, ByVal width, ByVal height)
    Dim pntsArray(2,5)
    
    pntsArray(0,0) = originX
    pntsArray(1,0) = originY
    pntsArray(2,0) = 0
    pntsArray(0,1) = originX
    pntsArray(1,1) = originY + height
    pntsArray(2,1) = 0
    pntsArray(0,2) = originX + width
    pntsArray(1,2) = originY + height
    pntsArray(2,2) = 0
    pntsArray(0,3) = originX + width
    pntsArray(1,3) = originY
    pntsArray(2,3) = 0
    pntsArray(0,4) = originX
    pntsArray(1,4) = originY
    pntsArray(2,4) = 0
    
    GetRectPntsArray = pntsArray
End Function







Function getpntcmd_OnTerminate()
	ClearMotionGraphics
	Set GetPointCmd = Nothing
End Function

Function set_units()

	Select Case doc.currentunit
		Case 2	'mils
			dielectricBoxX = 800   'width of the box for the dielectric layers
			dielectricBoxY = 100   'height of the box for the dielectric layers
			traceBoxX = 150        'width of the box for trace layers (two each layer)
			traceBoxY = 25         'height of the box for trace layers
			traceBox1OffsetX = 150 'start position of first trace box
			traceBox2OffsetX = 500 'start position of second trace box
			planeBoxX = 800        'width of the box for plane layers
			planeBoxY = 25         'height of the box for plane layers
			textOffsetX = 1000      'X offset for text for each layer
			textOffsetYdie = 50    'Y offset for text for dielectric layer
			textOffsetYsm = 5    'Y offset for text for solder mask layer
			textOffsetYcond = 5   'Y offset for text for conductive layers
			textHeight = 80        'height of text
			stepCondY = 250        'spacing above conductive layer
			stepDieY = 175         'spacing above dielectric layer
			stepTitleY = 150      'spacing in title area
			titleTextOffestX = 900 'start position of title text
			sm_box = 15			'height of the box for soldermask layers

		Case 3	'inches
			dielectricBoxX = .8   'width of the box for the dielectric layers
			dielectricBoxY = .1   'height of the box for the dielectric layers
			traceBoxX = .150        'width of the box for trace layers (two each layer)
			traceBoxY = .025         'height of the box for trace layers
			traceBox1OffsetX = .150 'start position of first trace box
			traceBox2OffsetX = .500 'start position of second trace box
			planeBoxX = .800        'width of the box for plane layers
			planeBoxY = .025         'height of the box for plane layers
			textOffsetX = 1      'X offset for text for each layer
			textOffsetYdie = .050    'Y offset for text for dielectric layer
			textOffsetYsm = .005    'Y offset for text for solder mask layer
			textOffsetYcond = .005   'Y offset for text for conductive layers
			textHeight = .080        'height of text
			stepCondY = .250        'spacing above conductive layer
			stepDieY = .175         'spacing above dielectric layer
			stepTitleY = .150      'spacing in title area
			titleTextOffestX = .900 'start position of title text
			sm_box = .015			'height of the box for soldermask layers		
		Case 4	'mm
			dielectricBoxX = 20.32   'width of the box for the dielectric layers
			dielectricBoxY = 2.54   'height of the box for the dielectric layers
			traceBoxX = 3.81        'width of the box for trace layers (two each layer)
			traceBoxY = 0.635         'height of the box for trace layers
			traceBox1OffsetX = 3.81 'start position of first trace box
			traceBox2OffsetX = 12.7 'start position of second trace box
			planeBoxX = 20.32        'width of the box for plane layers
			planeBoxY = 0.635         'height of the box for plane layers
			textOffsetX = 25.4      'X offset for text for each layer
			textOffsetYdie = 1.27    'Y offset for text for dielectric layer
			textOffsetYsm = .127    'Y offset for text for solder mask layer
			textOffsetYcond = .127   'Y offset for text for conductive layers
			textHeight = 2.032        'height of text
			stepCondY = 6.35        'spacing above conductive layer
			stepDieY = 4.445         'spacing above dielectric layer
			stepTitleY = 3.81      'spacing in title area
			titleTextOffestX = 22.86 'start position of title text
			sm_box = 0.381			'height of the box for soldermask layers		
		Case 5	'um
			dielectricBoxX = 20320   'width of the box for the dielectric layers
			dielectricBoxY = 2540   'height of the box for the dielectric layers
			traceBoxX = 3810        'width of the box for trace layers (two each layer)
			traceBoxY = 635         'height of the box for trace layers
			traceBox1OffsetX = 3810 'start position of first trace box
			traceBox2OffsetX = 12700 'start position of second trace box
			planeBoxX = 20320        'width of the box for plane layers
			planeBoxY = 635         'height of the box for plane layers
			textOffsetX = 25400     'X offset for text for each layer
			textOffsetYdie = 1270    'Y offset for text for dielectric layer
			textOffsetYsm = 127    'Y offset for text for solder mask layer
			textOffsetYcond = 127   'Y offset for text for conductive layers
			textHeight = 2032        'height of text
			stepCondY = 6350        'spacing above conductive layer
			stepDieY = 4445         'spacing above dielectric layer
			stepTitleY = 3810      'spacing in title area
			titleTextOffestX = 22860 'start position of title text
			sm_box = 381			'height of the box for soldermask layers		
	
	End Select


End Function


Public Sub get_id()
	dim shell
	dim local_app_data
	dim idx
	dim fso
	dim file
	dim tmp
	dim ary

	Set shell = CreateObject( "WScript.Shell" )
	'get appdata path
	local_app_data = shell.ExpandEnvironmentStrings("%APPDATA%")

	're-route appdata path to local\ExactCAD
	idx = instr(local_app_data, "AppData")

	'add the Selected_Version filename
	local_app_data = Left(local_app_data, idx + 6) & "\Local\ExactCAD\Selected_Version.txt"


	'create file system object
	Set fso = CreateObject("Scripting.FileSystemObject")

	'see if the file exists
	If (fso.FileExists(local_app_data)) Then
		'if so, open the file
		set file = fso.OpenTextFile(local_app_data)
		
		'read each line of the file
		Do until file.AtEndOfStream
			tmp = file.ReadLine
			
			'look for maglaunch_path in each line of the file
			if instr(tmp, vbtab) > 0 then
				
				'when found, split the string into an array using the tab delimiter
				ary = split(tmp, vbtab)
				
				if ary(0) = "id" then
					prog_id = ary(1)
				end if

			end if
		Loop
		
		'close the file
		file.Close
		
	End If

	'terminate the shell
	set shell = Nothing



End Sub


Public Function GetLicensedDoc(appObj)
    On Error Resume next
    Dim key,licenseServer,licenseToken,docObj

    Set GetLicensedDoc = Nothing

    ' collect the active document
    Set docObj = appObj.ActiveDocument
    If (Err) Then  
        Call appObj.Gui.StatusBarText("No active document: " + _
                                 Err.Description,epcbStatusFieldError)
        Exit Function
    End If

    ' Ask Expedition’s document for the key
    key = docObj.Validate(0)

    ' Get token from license server
    Set licenseServer = CreateObject("MGCPCBAutomationLicensing.Application")
    licenseToken = licenseServer.GetToken(key)
    Set licenseServer = Nothing

    ' Ask the document to validate the license token
    Err.Clear 
    docObj.Validate(licenseToken)
    If (Err) Then  
        Call appObj.Gui.StatusBarText("No active document license: " + _
                                      Err.Description,epcbStatusFieldError)
        Exit Function
    End If

    ' everything is OK, return document
    Set GetLicensedDoc = docObj 
End Function
